@@ -15,7 +15,8 @@ import numpy as np
 from scipy import sparse
 from numpy.lib.stride_tricks import as_strided
 
-from ..utils import check_array, check_random_state
+from ..utils import check_array, check_random_state, deprecated
+from ..utils.validation import _deprecate_positional_args
 from ..base import BaseEstimator
 
 __all__ = ['PatchExtractor',
@@ -33,11 +34,11 @@ def _make_edges_3d(n_x, n_y, n_z=1):
 
     Parameters
     ----------
-    n_x : integer
+    n_x : int
         The size of the grid in the x direction.
-    n_y : integer
+    n_y : int
         The size of the grid in the y direction.
-    n_z : integer, optional
+    n_z : integer, default=1
         The size of the grid in the z direction, defaults to 1
     """
     vertices = np.arange(n_x * n_y * n_z).reshape((n_x, n_y, n_z))
@@ -138,14 +139,16 @@ def img_to_graph(img, mask=None, return_as=sparse.coo_matrix, dtype=None):
 
     Parameters
     ----------
-    img : ndarray, 2D or 3D
-        2D or 3D image
-    mask : ndarray of booleans, optional
+    img : ndarray of shape (height, width) or (height, width, channel)
+        2D or 3D image.
+    mask : ndarray of shape (height, width) or \
+            (height, width, channel), dtype=bool, default=None
         An optional mask of the image, to consider only part of the
         pixels.
-    return_as : np.ndarray or a sparse matrix class, optional
+    return_as : np.ndarray or a sparse matrix class, \
+            default=sparse.coo_matrix
         The class to use to build the returned adjacency matrix.
-    dtype : None or dtype, optional
+    dtype : dtype, default=None
         The data of the returned sparse matrix. By default it is the
         dtype of img
 
@@ -175,14 +178,15 @@ def grid_to_graph(n_x, n_y, n_z=1, mask=None, return_as=sparse.coo_matrix,
         Dimension in x axis
     n_y : int
         Dimension in y axis
-    n_z : int, optional, default 1
+    n_z : int, default=1
         Dimension in z axis
-    mask : ndarray of booleans, optional
+    mask : ndarray of shape (n_x, n_y, n_z), dtype=bool, default=None
         An optional mask of the image, to consider only part of the
         pixels.
-    return_as : np.ndarray or a sparse matrix class, optional
+    return_as : np.ndarray or a sparse matrix class, \
+            default=sparse.coo_matrix
         The class to use to build the returned adjacency matrix.
-    dtype : dtype, optional, default int
+    dtype : dtype, default=int
         The data of the returned sparse matrix. By default it is int
 
     Notes
@@ -216,7 +220,7 @@ def _compute_n_patches(i_h, i_w, p_h, p_w, max_patches=None):
         The height of a patch
     p_w : int
         The width of a patch
-    max_patches : integer or float, optional default is None
+    max_patches : int or float, default=None
         The maximum number of patches to extract. If max_patches is a float
         between 0 and 1, it is taken to be a proportion of the total number
         of patches.
@@ -241,7 +245,7 @@ def _compute_n_patches(i_h, i_w, p_h, p_w, max_patches=None):
         return all_patches
 
 
-def extract_patches(arr, patch_shape=8, extraction_step=1):
+def _extract_patches(arr, patch_shape=8, extraction_step=1):
     """Extracts patches of any n-dimensional array in place using strides.
 
     Given an n-dimensional array it will return a 2n-dimensional array with
@@ -257,12 +261,12 @@ def extract_patches(arr, patch_shape=8, extraction_step=1):
     arr : ndarray
         n-dimensional array of which patches are to be extracted
 
-    patch_shape : integer or tuple of length arr.ndim
+    patch_shape : int or tuple of length arr.ndim.default=8
         Indicates the shape of the patches to be extracted. If an
         integer is given, the shape will be a hypercube of
         sidelength given by its value.
 
-    extraction_step : integer or tuple of length arr.ndim
+    extraction_step : int or tuple of length arr.ndim, default=1
         Indicates step size at which extraction shall be performed.
         If integer is given, then the step is uniform in all dimensions.
 
@@ -299,6 +303,47 @@ def extract_patches(arr, patch_shape=8, extraction_step=1):
     return patches
 
 
+@deprecated("The function feature_extraction.image.extract_patches has been "
+            "deprecated in 0.22 and will be removed in 0.24.")
+def extract_patches(arr, patch_shape=8, extraction_step=1):
+    """Extracts patches of any n-dimensional array in place using strides.
+
+    Given an n-dimensional array it will return a 2n-dimensional array with
+    the first n dimensions indexing patch position and the last n indexing
+    the patch content. This operation is immediate (O(1)). A reshape
+    performed on the first n dimensions will cause numpy to copy data, leading
+    to a list of extracted patches.
+
+    Read more in the :ref:`User Guide <image_feature_extraction>`.
+
+    Parameters
+    ----------
+    arr : ndarray
+        n-dimensional array of which patches are to be extracted
+
+    patch_shape : int or tuple of length arr.ndim, default=8
+        Indicates the shape of the patches to be extracted. If an
+        integer is given, the shape will be a hypercube of
+        sidelength given by its value.
+
+    extraction_step : int or tuple of length arr.ndim, default=1
+        Indicates step size at which extraction shall be performed.
+        If integer is given, then the step is uniform in all dimensions.
+
+
+    Returns
+    -------
+    patches : strided ndarray
+        2n-dimensional array indexing patches on first n dimensions and
+        containing patches on the last n dimensions. These dimensions
+        are fake, but this way no data is copied. A simple reshape invokes
+        a copying operation to obtain a list of patches:
+        result.reshape([-1] + list(patch_shape))
+    """
+    return _extract_patches(arr, patch_shape=patch_shape,
+                            extraction_step=extraction_step)
+
+
 def extract_patches_2d(image, patch_size, max_patches=None, random_state=None):
     """Reshape a 2D image into a collection of patches
 
@@ -308,29 +353,28 @@ def extract_patches_2d(image, patch_size, max_patches=None, random_state=None):
 
     Parameters
     ----------
-    image : array, shape = (image_height, image_width) or
+    image : ndarray of shape (image_height, image_width) or \
         (image_height, image_width, n_channels)
         The original image data. For color images, the last dimension specifies
         the channel: a RGB image would have `n_channels=3`.
 
-    patch_size : tuple of ints (patch_height, patch_width)
-        the dimensions of one patch
+    patch_size : tuple of int (patch_height, patch_width)
+        The dimensions of one patch.
 
-    max_patches : integer or float, optional default is None
-        The maximum number of patches to extract. If max_patches is a float
+    max_patches : int or float, default=None
+        The maximum number of patches to extract. If `max_patches` is a float
         between 0 and 1, it is taken to be a proportion of the total number
         of patches.
 
-    random_state : int, RandomState instance or None, optional (default=None)
-        Pseudo number generator state used for random sampling to use if
-        `max_patches` is not None.  If int, random_state is the seed used by
-        the random number generator; If RandomState instance, random_state is
-        the random number generator; If None, the random number generator is
-        the RandomState instance used by `np.random`.
+    random_state : int, RandomState instance, default=None
+        Determines the random number generator used for random sampling when
+        `max_patches` is not None. Use an int to make the randomness
+        deterministic.
+        See :term:`Glossary <random_state>`.
 
     Returns
     -------
-    patches : array, shape = (n_patches, patch_height, patch_width) or
+    patches : array of shape (n_patches, patch_height, patch_width) or \
         (n_patches, patch_height, patch_width, n_channels)
         The collection of patches extracted from the image, where `n_patches`
         is either `max_patches` or the total number of patches that can be
@@ -348,12 +392,12 @@ def extract_patches_2d(image, patch_size, max_patches=None, random_state=None):
     >>> print('Patches shape: {}'.format(patches.shape))
     Patches shape: (272214, 2, 2, 3)
     >>> # Here are just two of these patches:
-    >>> print(patches[1]) # doctest: +NORMALIZE_WHITESPACE
+    >>> print(patches[1])
     [[[174 201 231]
       [174 201 231]]
      [[173 200 230]
       [173 200 230]]]
-    >>> print(patches[800])# doctest: +NORMALIZE_WHITESPACE
+    >>> print(patches[800])
     [[[187 214 243]
       [188 215 244]]
      [[187 214 243]
@@ -374,9 +418,9 @@ def extract_patches_2d(image, patch_size, max_patches=None, random_state=None):
     image = image.reshape((i_h, i_w, -1))
     n_colors = image.shape[-1]
 
-    extracted_patches = extract_patches(image,
-                                        patch_shape=(p_h, p_w, n_colors),
-                                        extraction_step=1)
+    extracted_patches = _extract_patches(image,
+                                         patch_shape=(p_h, p_w, n_colors),
+                                         extraction_step=1)
 
     n_patches = _compute_n_patches(i_h, i_w, p_h, p_w, max_patches)
     if max_patches:
@@ -406,20 +450,20 @@ def reconstruct_from_patches_2d(patches, image_size):
 
     Parameters
     ----------
-    patches : array, shape = (n_patches, patch_height, patch_width) or
+    patches : ndarray of shape (n_patches, patch_height, patch_width) or \
         (n_patches, patch_height, patch_width, n_channels)
         The complete set of patches. If the patches contain colour information,
         channels are indexed along the last dimension: RGB patches would
         have `n_channels=3`.
 
-    image_size : tuple of ints (image_height, image_width) or
+    image_size : tuple of int (image_height, image_width) or \
         (image_height, image_width, n_channels)
-        the size of the image that will be reconstructed
+        The size of the image that will be reconstructed.
 
     Returns
     -------
-    image : array, shape = image_size
-        the reconstructed image
+    image : ndarray of shape image_size
+        The reconstructed image.
     """
     i_h, i_w = image_size[:2]
     p_h, p_w = patches.shape[1:3]
@@ -444,21 +488,23 @@ class PatchExtractor(BaseEstimator):
 
     Read more in the :ref:`User Guide <image_feature_extraction>`.
 
+    .. versionadded:: 0.9
+
     Parameters
     ----------
-    patch_size : tuple of ints (patch_height, patch_width)
-        the dimensions of one patch
+    patch_size : tuple of int (patch_height, patch_width)
+        The dimensions of one patch.
 
-    max_patches : integer or float, optional default is None
+    max_patches : int or float, default=None
         The maximum number of patches per image to extract. If max_patches is a
         float in (0, 1), it is taken to mean a proportion of the total number
         of patches.
 
-    random_state : int, RandomState instance or None, optional (default=None)
-        If int, random_state is the seed used by the random number generator;
-        If RandomState instance, random_state is the random number generator;
-        If None, the random number generator is the RandomState instance used
-        by `np.random`.
+    random_state : int, RandomState instance, default=None
+        Determines the random number generator used for random sampling when
+        `max_patches` is not None. Use an int to make the randomness
+        deterministic.
+        See :term:`Glossary <random_state>`.
 
     Examples
     --------
@@ -474,21 +520,22 @@ class PatchExtractor(BaseEstimator):
     >>> print('Patches shape: {}'.format(pe_trans.shape))
     Patches shape: (545706, 2, 2)
     """
-
-    def __init__(self, patch_size=None, max_patches=None, random_state=None):
+    @_deprecate_positional_args
+    def __init__(self, *, patch_size=None, max_patches=None,
+                 random_state=None):
         self.patch_size = patch_size
         self.max_patches = max_patches
         self.random_state = random_state
 
     def fit(self, X, y=None):
-        """Do nothing and return the estimator unchanged
+        """Do nothing and return the estimator unchanged.
 
         This method is just there to implement the usual API and hence
         work in pipelines.
 
         Parameters
         ----------
-        X : array-like, shape [n_samples, n_features]
+        X : array-like of shape (n_samples, n_features)
             Training data.
         """
         return self
@@ -498,7 +545,7 @@ class PatchExtractor(BaseEstimator):
 
         Parameters
         ----------
-        X : array, shape = (n_samples, image_height, image_width) or
+        X : ndarray of shape (n_samples, image_height, image_width) or \
             (n_samples, image_height, image_width, n_channels)
             Array of images from which to extract patches. For color images,
             the last dimension specifies the channel: a RGB image would have
@@ -506,7 +553,7 @@ class PatchExtractor(BaseEstimator):
 
         Returns
         -------
-        patches : array, shape = (n_patches, patch_height, patch_width) or
+        patches : array of shape (n_patches, patch_height, patch_width) or \
              (n_patches, patch_height, patch_width, n_channels)
              The collection of patches extracted from the images, where
              `n_patches` is either `n_samples * max_patches` or the total
